// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
/* eslint-disable eqeqeq */
import { logger } from "./log";
/**
 * Describes the operations that can be performed on the message header.
 * @module MessageHeader
 */
export const MessageHeader = {
    /**
     * Converts MessageHeader to AmqpMessageHeader.
     *
     * @param {MessageHeader} props Message header.
     * @returns {AmqpMessageHeader} AmqpMessageHeader
     */
    toAmqpMessageHeader(props) {
        const amqpHeader = {};
        if (props.deliveryCount != undefined) {
            amqpHeader.delivery_count = props.deliveryCount;
        }
        if (props.durable != undefined)
            amqpHeader.durable = props.durable;
        if (props.firstAcquirer != undefined) {
            amqpHeader.first_acquirer = props.firstAcquirer;
        }
        if (props.priority != undefined) {
            amqpHeader.priority = props.priority;
        }
        if (props.ttl != undefined) {
            amqpHeader.ttl = props.ttl;
        }
        logger.verbose("To AmqpMessageHeader: %O", amqpHeader);
        return amqpHeader;
    },
    /**
     * Converts AmqpMessageHeader to MessageHeader.
     *
     * @param {AmqpMessageHeader} props Amqp Message Header
     * @returns {MessageHeader} MessageHeader.
     */
    fromAmqpMessageHeader(props) {
        const msgHeader = {};
        if (props.delivery_count != undefined) {
            msgHeader.deliveryCount = props.delivery_count;
        }
        if (props.durable != undefined) {
            msgHeader.durable = props.durable;
        }
        if (props.first_acquirer != undefined) {
            msgHeader.firstAcquirer = props.first_acquirer;
        }
        if (props.priority != undefined) {
            msgHeader.priority = props.priority;
        }
        if (props.ttl != undefined) {
            msgHeader.ttl = props.ttl;
        }
        logger.verbose("From AmqpMessageHeader: %O", msgHeader);
        return msgHeader;
    }
};
//# sourceMappingURL=messageHeader.js.map